/*
  WN Reading Device Code
  Adapted from OLED_featherwing.ino from Adafruit's library
  
*/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SH110X.h>

Adafruit_SH1107 display = Adafruit_SH1107(64, 128, &Wire);



#define BUTTON_A  9
#define BUTTON_B  8
#define BUTTON_C  7

char msgBuf[64]; 
uint8_t retVal;


void setup() {
  display.begin(0x3C, true); // Address 0x3C default
  display.display();

  // Clear the buffer
  display.clearDisplay();
  display.display();
  display.setRotation(1);  


  pinMode(BUTTON_A, INPUT_PULLUP);
  pinMode(BUTTON_B, INPUT_PULLUP);
  pinMode(BUTTON_C, INPUT_PULLUP);

  display.setTextSize(2);
  display.setTextColor(SH110X_WHITE);
  display.setCursor(0,0); 

  display.print("starting...");
  display.display();  

  Wire.setSDA(2);
  Wire.setSCL(3);
  Wire.begin(); //join as master device
  Wire.setTimeout(1000);
  
}

void loop() {
  delay(1000);
  display.clearDisplay();
  display.setCursor(0,0); 

  uint8_t dataLen = 64;
  char dataBuf[dataLen];
  readFromEEPROM(dataBuf, dataLen);
  for(uint8_t i=0; i < dataLen; i++){
    if(dataBuf[i]=='$') {break;}
    if(dataBuf[i]>127)  {
      displayReadError();
      break;
    }
    display.print(String(dataBuf[i]));
  }
  display.display(); //updates the display
}


//caller must check that len won't read past end of retBuf
void readFromEEPROM(char * retBuf, uint8_t len){ //buffer for ret bytes, number of bytes to be read
  //read the datasheet for sequential reading on 24LC256 to see why this works
  byte memAddr[2] = {0}; //hardcoded bc we're never going to read anywhere else. Hard to pass 2 bytes to a function anyway
  Wire.beginTransmission(0x50);
  Wire.write(memAddr, 2); //THIS IS ALWAYS 2 BC IT'S THE NUM OF ADDR BYTES
  Wire.endTransmission(false); //must be false to avoid sending stop condition
  uint8_t readBytes = 0; //can use this to check for errors. implement later.
  readBytes = Wire.requestFrom(0x50, len, true);
  //now put the returned bytes into the buffer
  for (uint8_t i = 0; i < readBytes; ++i) {
    retBuf[i] = Wire.read();
  }
}

//prints error in reading which happens when we read junk, i.e. ASCII characters greater than 127
void displayReadError(){
  display.clearDisplay();
  display.println("READ ERROR\n    :(");
}
